#ifndef	_Asymmetry_h_
#define	_Asymmetry_h_

#include <TNamed.h>
#include <Riostream.h>
#include "ChGroup.h"

const Int_t tbin = 64000;

class Asymmetry : public TNamed {
private:
    GroupInfo * fStateA; // !
    GroupInfo * fStateB; // !
    Int_t * fA;
    Int_t * fB;
    TH1 * fHist;
public:
    Asymmetry(ChGroup * a, ChGroup * b) {
      //      printf("Entering Asymmetry");
	assert(a);
	assert(b);
	fStateA = a->GetInfo();
	fStateB = b->GetInfo();
	fA = new Int_t[tbin];
	fB = new Int_t[tbin];

	TString name("Asym_");
	name += a->GetName();
	name += "_vs_";
	name += b->GetName();

	TString title("Asym of ");
	title += a->GetName();
	title += " vs ";
	title += b->GetName();

	SetNameTitle(name, title);

	fHist = new TH1D(name, title, tbin, 0, tbin);

	Update();
	//      printf("Exitting Asymmetry");
    };

    ~Asymmetry() {
	delete [] fA;
	delete [] fB;
	delete fHist;
    };

    void Fill(Int_t ch, Double_t time) {
	if (fStateA->GetCh(ch))
	    fA[(Int_t)time]++;
	if (fStateB->GetCh(ch))
	    fB[(Int_t)time]++;
    };

    void Reset() {
	memset(fA, 0, sizeof(fA[0]) * tbin);
	memset(fB, 0, sizeof(fB[0]) * tbin);
	fHist->Reset();
    };

    void Update() {
cout << endl;
cout << "......." << fHist->GetName() << " Update" << endl;
cout << "tbin=" << tbin << endl;
cout << endl;
	fHist->Reset();
	fHist->GetYaxis()->SetRangeUser(-1,1);
	for (int i=0; i<tbin; i++) {
	    if (fA[i] || fB[i]) {
// cout << fHist->GetName() << ' ' << i << ' ' << fA[i] << ' ' << fB[i] << endl;
		fHist->Fill(i, (Double_t)(fA[i]-fB[i])/(Double_t)(fA[i]+fB[i]));
	    }
	}
    };

    void Draw(Option_t * opt = "") {
	fHist->Draw(opt);
    };

    ClassDef(Asymmetry,0)
};

#endif
