#include "Coincidence2.h"

#include "Hit.h"
#include "CoincidencedHit.h"
#include <assert.h>
#include <vector>
#include <algorithm>
#include <TCollection.h>

class timeinfo {
public:
    int time;
    int op;	/* +1: outer hit opens gate */
		/* -1: outer hit closes gate */
		/*  0: gate state is not changed (used by innter hit) */
};

class timeinfo_cmp_functor {
public:
    bool operator()(const timeinfo & a, const timeinfo & b) const {
	if (b.time > a.time)
	    return true;

	if (b.time == a.time) {
	    if (a.op > b.op)
		return true;
	}

	return false;
    };
};

void
Coincidence2(TCollection & inner_list, TCollection & outer_list, Int_t gap, TCollection & result)
{

    const Int_t w = gap / 2;
    const Int_t n_inner(inner_list.GetEntries());
    const Int_t n_outer(outer_list.GetEntries());

    if (n_inner == 0)
	return;

    if (n_outer == 0)
	return;

    Int_t vector_size = n_inner + n_outer * 2;

    std::vector<timeinfo> t(vector_size);

    int i = 0;

    int inner_ch = -1, outer_ch = -1;

    TIter inner_iter(&inner_list);

    while (TObject * o = inner_iter()) {
	assert(o->InheritsFrom(Hit::Class()));
	Hit * h = (Hit *)o;

	if (inner_ch == -1)
	    inner_ch = h->GetCh();

	t[i].time = h->GetTime();
	t[i].op = 0;

	i++;
    }


    TIter outer_iter(&outer_list);

    while (TObject * o = outer_iter()) {
	assert(o->InheritsFrom(Hit::Class()));
	Hit * h = (Hit *)o;

	if (outer_ch == -1)
	    outer_ch = h->GetCh();

	t[i].time = h->GetTime() - w;
	t[i].op = 1;	/* Gate Open */
	i++;

	t[i].time = h->GetTime() + w;
	t[i].op = -1;	/* Gate Close */
	i++;
    }

    assert(i == vector_size);

    std::sort(t.begin(), t.end(), timeinfo_cmp_functor());

    result.SetOwner(kTRUE);

    int state = 0;

    for (int i=0; i<vector_size; i++) {
	Int_t last_outer_time;

	switch (t[i].op) {
	case +1: /* Gate Open */
	    last_outer_time = t[i].time + w;
	case -1: /* Gate Close */
	    state += t[i].op;
	    break;
	case 0: /* Inner hit */
	    if (state > 0) { /* If Gate is opened */
		Hit inner(inner_ch, t[i].time);
		Hit outer(outer_ch, last_outer_time);
		result.Add(new CoincidencedHit(inner, outer));
	    }
	    break;
	}

    }
}
