#ifndef	_EDB_H_
#define	_EDB_H_
#include <Riostream.h>
#include <TDatime.h>
#include <TString.h>

#include <sys/stat.h> /* for mkdir */
#include <errno.h>
#include <assert.h>
#include <dirent.h> /* for opendir */

class EDB {
private:

    /* base directory, like a /home/daq/Data */
    TString fBaseDir;

    /* run directory, like a /home/daq/Data/MUSE011113_xxxx */
    TString fRunDir;

    /* filename, like a */
    TString fFileName;

    FILE * fFile;
    Int_t fRunNumber;
    Int_t fFileNumber;
    Int_t fWrittenBytes; /* Written data size for current file */

    /* Utility Functions */

    /* dirname only date */
    TString
    muse_dirname1(int run_number)
    {
	TDatime now;
	TString dirname;
	dirname.Form("MUSE%06d_%08d", run_number, now.GetDate());
	return dirname;
    };

    /* dirname date + time */
    TString
    muse_dirname2(int run_number)
    {
	TDatime now;
	TString dirname;
	dirname.Form("MUSE%06d_%08d_%06d",
		run_number, now.GetDate(), now.GetTime());
	return dirname;
    };

    TString
    generate_directory(const char * basedir, int run_number)
    {
	TString p;
	int ret;

	p = basedir; p += "/"; p += muse_dirname1(run_number);
	ret = mkdir(p, 0755);
	if (ret == 0)
	    return p;

	if (errno != EEXIST)
	    abort();

	p = basedir; p += "/"; p += muse_dirname2(run_number);
	ret = mkdir(p, 0755);
	if (ret == 0)
	    return p;

	abort();
    };

public:
    EDB(const char * basedir) :
	fBaseDir(basedir),
	fRunNumber(-1),
	fFile(NULL),
	fFileNumber(0),
	fWrittenBytes(0) {
	  printf("testing: %s",basedir);
	DIR * dtest = opendir(basedir);
	assert(dtest);
	closedir(dtest);
    };

    void
    EndRun() {
	if (fFile)
	    fclose(fFile);
	fFile = NULL;
    };

    ~EDB() {
	EndRun();
    };

    TString
    FileName(Int_t fFileNumber) {
	TString p;
	p.Form("MUSE%06d_01_001_%03d.edb",
	    fRunNumber, fFileNumber);
	return p;
    };

    void
    NewFile() {
	if (fFile)
	    fclose(fFile);

	fFile = NULL;

	fFileName = FileName(fFileNumber);

	cout << fRunDir << endl;
	cout << fFileName << endl;

	cout << fRunDir + "/" + fFileName << endl;

	fFile = fopen(fRunDir + "/" + fFileName, "w");
	assert(fFile);

	fWrittenBytes = 0;
    };

    TString CurrentFilename() const {
	return fRunDir + "/" + fFileName;
    };

    void
    NewRun(int run_number) {
	EndRun();
	assert(run_number > 0);
	fRunNumber = run_number;
	assert(fBaseDir != "");

	fFileNumber = 0;

	fRunDir = generate_directory(fBaseDir, fRunNumber);
	NewFile();
    };

    void
    ChangeFile() {
	fFileNumber++;
	NewFile();
    };

    void
    WriteData(const void * buffer, Int_t nbytes) {
	if (fFile == NULL)
	    return;

	int ret = fwrite(buffer, nbytes, 1, fFile);
	if (ret != 1)
	    fprintf(stderr, "ferror = %d %s\n", ferror(fFile), strerror(ferror(fFile)));

	fWrittenBytes += nbytes;

	if (fWrittenBytes > 1024 * 1024 * 1024)
	    ChangeFile();
    };

};
#endif
