#ifndef _GathererDecoder_h_
#define _GathererDecoder_h_

#include <TObjArray.h>

// #include "FPGATDC32decoder.h"
// #include "rawevent.h"

#include "FPGATDC32.h"
#include "MLFdatum.h"

#include "copper_lite.h"
#include "kalliope_tdc.h"
#include "niki_3n00.h"

#define	NW(X)	(sizeof(X)/4)

#define	MAX_NHIT_PER_CHANNEL	200

class GathererDecoder {
private:
    Int_t fSlot;
    Int_t niki_event_id_save;
    Int_t fN;
    Int_t fMaxhit;
    Int_t fOverFlow;
    MLFdatum * fBuffer; // [fMaxhit]
public:
    GathererDecoder(Int_t maxhit = 1024 * 256) :
	fSlot(0),
	  niki_event_id_save(-1),
	fN(0),
	fMaxhit(maxhit),
	fBuffer(new MLFdatum[fMaxhit]) { };

    virtual ~GathererDecoder() {
	delete [] fBuffer;
    };

    virtual void OverFlowHandler(const FPGATDC32hit & raw) {
	fOverFlow++;
    };

    virtual void Push(Int_t ch, Int_t time, const FPGATDC32hit & raw) {
	fBuffer[fN++].SetTDC(fSlot, ch, time);
    };

    virtual void Push(Long64_t val) {
	fBuffer[fN++].Set64(val);
    };

    virtual void Push(MLFdatum v) {
	fBuffer[fN++] = v;
	//	printf("fHeader %X\n",v.GetType());
    };

    void Reset() {
	fOverFlow = 0;
	fN = 0;
    };

    Int_t Truncate(Int_t upto) {
	if (fN > upto) {
	    fN = upto;
	}
	return fN;
    };

    Int_t GetN() { return fN; };
    Int_t GetNbytes() { return fN * 8; };

    Int_t IsOverFlowed() { return fOverFlow; };

    void FillBuffer(UChar_t * buf, Int_t nbytes) {
	memcpy(buf, fBuffer, nbytes);
    };

    const UChar_t * const GetBuffer() const {
	return (const UChar_t * const)fBuffer;
    };

    void decode_finesse(const UInt_t * const p, Int_t nword, Int_t fslot) throw(char const *) {
	const finheader * const finhdr = (const finheader * const)p;
	const finfooter * const finftr = (const finfooter * const)(p+nword-NW(finfooter));

	if (finhdr->magic != 0xFFAA0000) {
	    throw("Bad Header Magic != 0xFFAA0000");
	}

	if ((finftr->magic & 0xFFFF0000) != 0xFF550000) {
	    throw("Bad Footer Magic != 0xFF55XXXX");
	}

	FPGATDC32hit * q = (FPGATDC32hit *)(&finhdr[1]);
	int nhit = nword - NW(*finhdr) - NW(*finftr);
	UShort_t start = finftr->start;
#ifdef MAX_NHIT_PER_CHANNEL	
	int nhit_on_ch[32];
	memset(nhit_on_ch, 0, sizeof(nhit_on_ch));
#endif
	for (int i=0; i<nhit; i++) {
	    if (q[i].IsOverflowed())
		OverFlowHandler(q[i]);

	    int qch = q[i].GetCh();
	    Int_t ch = q[i].GetCh()+32*fslot;
	    Int_t time = (q[i].time - start)&0xFFFF;

#ifdef MAX_NHIT_PER_CHANNEL
	    if (nhit_on_ch[qch] < MAX_NHIT_PER_CHANNEL) {
		Push(ch, time, q[i]);
	    }
	    ++nhit_on_ch[qch];
#else
	    Push(ch, time, q[i]);
#endif
	}

#ifdef MAX_NHIT_PER_CHANNEL
	for (int i=0; i<32; i++) {
	    if (nhit_on_ch[i] >= MAX_NHIT_PER_CHANNEL) {
	      fprintf(stderr, "fslot=%d ch=%d nhit=%d >= %d\n", fslot,q[i].GetCh(),nhit_on_ch[i],MAX_NHIT_PER_CHANNEL);
	    }
	}
#endif
    };


    void decode_niki(const niki_3n00_event & e, int module_id) throw (char const *) {
	int resolution = e.header().resolution();
	int nhit = e.nhit();

	Long64_t start = e.start();
	//	const niki_stop_data * p = (niki_stop_data *)e.body();
	const niki_stop_data * p = (niki_stop_data *)e.hits();

#ifdef	MAX_NHIT_PER_CHANNEL
	int nhit_on_ch[32];
	memset(nhit_on_ch, 0, sizeof(nhit_on_ch));
#endif


	for (int hit=0; hit<nhit; hit++) {
	    int module = -1;
	    int ch = -1;

	    Int_t adc_count = 0;
	    Long64_t tdc_count = 0;
	    for (int i=0; i<4; i++) {
		const niki_stop_data & raw = p[4*hit+i];

		if (ch == -1) {
		    ch = raw.GetCh();
		} else {
		    assert(ch == raw.GetCh());
		}

		if (module == -1) {
		    module = raw.GetModule();
		}

		switch (raw.What()) {
		case 1: /* TDC 15:0 */
		    tdc_count |= raw.GetTDC();
		    break;
		case 2: /* TDC 31:16 */
		    tdc_count |= (raw.GetTDC() << 16);
		    break;
		case 3: /* TDC 39:32 */
		    tdc_count |= (((Long64_t)raw.GetTDC() & 0xFF) << 32);
		    break;
		case 4: /* ADC */
		    adc_count = raw.GetTDC();
		    break;
		default:
		    throw("unexpected StopID");
		}
	    }

	    // TDC time data from NIKI module is the time difference from start */
	    // tdc_count -= e.footer().start(); // is not necesasry

	    MLFdatum niki_adc_hit, niki_tdc_hit, niki_hit;
	    //	    	    printf("adc: %ld %ld %ld\n",module, ch, adc_count);
	    //	    	    printf("tdc: %ld %ld %ld\n",module, ch, tdc_count);//here comes adc OK
	    niki_hit.SetNikiTDCADC(module,ch,resolution,tdc_count,adc_count);
	    niki_adc_hit.SetNikiADC(module, ch, adc_count);
	    niki_tdc_hit.SetNikiTDC(module, ch, resolution, tdc_count);

	    assert(niki_tdc_hit.GetResolution() == resolution);

#ifdef	MAX_NHIT_PER_CHANNEL
	    assert(ch < 32);
	    if (nhit_on_ch[ch] < MAX_NHIT_PER_CHANNEL) {
		++nhit_on_ch[ch];
		if(USE_46PACKET) Push(niki_hit);
		else{
		  Push(niki_tdc_hit);
		  Push(niki_adc_hit);//push is OK
		}
	    }
#else
	    if(USE_46PACKET) Push(niki_hit);
	    else{
	      Push(niki_tdc_hit);
	      Push(niki_adc_hit);//push is OK
	    }
	    //Push(niki_hit);
	    //	    Push(niki_tdc_hit);
	    //	    Push(niki_adc_hit);//push is OK
#endif
	}
    };

    void decode_copper_lite_event(const copper_lite_event & e, Int_t copper_slot) {
	fSlot = copper_slot;
	fBuffer[fN++].Set32(htonl(e.header().id())); /* event counter */
	fBuffer[fN++].Set32(htonl(e.header().finesse())); /* finesse slot */
	fBuffer[fN++].Set32(htonl(e.header().length())); /* nbytes */
	decode_finesse((UInt_t *)e.body(), e.len()/4, e.header().finesse()>>24);
    };

    void decode_kalliope_tdc_event(const kalliope_tdc_event & e, Int_t module_id) {
	fSlot = module_id;
	fBuffer[fN++].Set32(htonl(e.header().event_id()));
	fBuffer[fN++].Set32(htonl(e.header().module_id()));
	fBuffer[fN++].Set32(htonl(e.header().length()));
	const finfooter * const finftr = (const finfooter * const)((UInt_t *)e.body()+e.len()/4-sizeof(finfooter)/4); //!DC
	fBuffer[fN++].Set32(htonl(finftr->start));                                                //!DC
	fBuffer[fN++].Set32(htonl(e.footer().value()));                                                //!DC
	fBuffer[fN++].Set32(htonl(module_id));
	decode_finesse((UInt_t *)e.body(), e.len()/4, 0);//e.header().module_id());
    };

    void decode_nim_tdc_event(const kalliope_tdc_event & e, Int_t module_id) {
	fSlot = module_id;
	fBuffer[fN++].Set32N(htonl(e.header().event_id()),0x10);
	fBuffer[fN++].Set32N(htonl(e.header().module_id()),0x11);
	fBuffer[fN++].Set32N(htonl(e.header().length()),0x10);
	const finfooter * const finftr = (const finfooter * const)((UInt_t *)e.body()+e.len()/4-sizeof(finfooter)/4); //!DC
	fBuffer[fN++].Set32N(htonl(finftr->start),0x10);                                                //!DC
	fBuffer[fN++].Set32N(htonl(e.footer().value()),0x10);                                                //!DC
	decode_finesse((UInt_t *)e.body(), e.len()/4, 0); //e.header().module_id());
    };

    void decode_niki_3n00_event(const niki_3n00_event & e, Int_t module_id) {
	fSlot = module_id;
	if(niki_event_id_save!=e.header().event_id()){
	  fBuffer[fN++].Set32N(htonl(e.header().event_id()),0x40);
	  fBuffer[fN++].Set32N(htonl(e.header().keyword()),0x40);
	  fBuffer[fN++].Set32N(htonl(e.header().length()),0x40);
	}
	niki_event_id_save=e.header().event_id();
	decode_niki(e, module_id);
    };

};
#endif
