#ifndef	_GroupInfo_h_
#define	_GroupInfo_h_

#include <assert.h>
#include <TNamed.h>

class GroupInfo : public TNamed {
private:
    Int_t fN;
    Int_t * fState; // [fN];
    Double_t * fZero;
    Double_t * fGain;
    Double_t * fUpper;
    Double_t * fLower;
    
public:
 GroupInfo(const char * name, const char * title, Int_t nch) :
    TNamed(name, title),
      fN(nch), 
      fState(new Int_t[fN]),
      fZero(new Double_t[fN]),
      fGain(new Double_t[fN]),
      fUpper(new Double_t[fN]),
      fLower(new Double_t[fN]) {
      //      printf("at GroupInfo: %d\n",nch);
      memset(fState, 0, sizeof(fState[0]) * fN);
      memset(fZero, 0, sizeof(fZero[0]) * fN);
      memset(fZero, 0, sizeof(fGain[0]) * fN);
      memset(fUpper, 0, sizeof(fUpper[0]) * fN);
      memset(fLower, 0, sizeof(fLower[0]) * fN);
    };
    

    ~GroupInfo() {
	delete [] fState;
	delete [] fZero;
	delete [] fGain;
	delete [] fUpper;
	delete [] fLower;
    };

    void AddCh(Int_t ch) {
	assert(ch >= 0);
	assert(ch < fN);
	fState[ch] = 1;
    };

    void SetCutRange(Int_t ch, Double_t lower, Double_t upper) {
	assert(ch >= 0);
	assert(ch < fN);
	fUpper[ch] = upper;
	fLower[ch] = lower;
    };

    Double_t GetUpper(Int_t ch){
	assert(ch >= 0);
	assert(ch < fN);
	return fUpper[ch];
    };

    Double_t GetLower(Int_t ch){
	assert(ch >= 0);
	assert(ch < fN);
	return fLower[ch];
    };

    void SetZeroGain(Int_t ch, Double_t zero, Double_t gain) {
	assert(ch >= 0);
	assert(ch < fN);
	fZero[ch] = zero;
	fGain[ch] = gain;
    };

    Double_t GetZero(Int_t ch){
	assert(ch >= 0);
	assert(ch < fN);
	return fZero[ch];
    };

    Double_t GetGain(Int_t ch){
	assert(ch >= 0);
	assert(ch < fN);
	return fGain[ch];
    };

    void DelCh(Int_t ch) {
	assert(ch >= 0);
	assert(ch < fN);
	fState[ch] = 0;
    };

    Int_t GetCh(Int_t ch) {
	assert(ch >= 0);
	assert(ch < fN);
	return fState[ch];
    };
    ClassDef(GroupInfo, 0)
};
#endif
