#ifndef	_copper_lite_h_
#define	_copper_lite_h_

#include <assert.h>
#include <string.h>
#include <unistd.h>
#include <stdio.h>

#include <netdb.h>

#include <queue>

#include <iostream>

#include "where.h"

#define	EVENT_BUFFER_SIZE	4*1024*1024

class copper_lite_header {
public:
    unsigned int m_magic;
    unsigned int m_finesse;
    unsigned int m_length;
    unsigned int m_id;

    unsigned int magic() const {
	return m_magic;
    };

    unsigned int finesse() const {
	return m_finesse;
    };

    unsigned int length() const {
	return m_length;
    };

    unsigned int id() const {
	return m_id;
    };

    void show() const {
	printf("magic = %08x\n", magic());
	printf("finesse = %08x\n", finesse());
	printf("length = %08x\n", length());
	printf("id = %08x\n", id());
    };
};

class finesse {
private:
    const unsigned char * const m_p8;
    const unsigned int * const m_p32;
    const int m_len;
public:
    finesse(unsigned char *p, int len) :
	m_p8(p),
	m_p32((const unsigned int * const)p),
	m_len(len) {
    };

};

class copper_lite_event {
private:
    copper_lite_header m_header;
    unsigned char *m_body;
    int m_footer;
    int m_len;

public:
    copper_lite_event() {
	m_body = new unsigned char [EVENT_BUFFER_SIZE];
    };

    void
    fill(copper_lite_header & header, int footer, unsigned char * p, int len) {
	m_header = header;
	m_footer = footer;
	m_len = len;
	memcpy(m_body, p, len);
    };

    copper_lite_event(const copper_lite_event & e) {
	m_header = e.m_header;
	m_len = e.m_len;
	m_footer = e.m_footer;

	m_body = new unsigned char [EVENT_BUFFER_SIZE];

	memcpy(m_body, e.m_body, m_len);
    };

    ~copper_lite_event() {
	delete [] m_body;
    };

    void show() const {

	printf("=============copper_lite_event.show()=======================\n");

	m_header.show();
	printf("footer = %08x\n", m_footer);

	assert(m_len % 4 == 0);

	int wlen = m_len / 4;

	unsigned int * p = (unsigned int *)m_body;

	for (int i=0; i<wlen; i++) {
	    if (i % 4 == 0)
		printf("%04x", i*4);
	    printf(" %08x", p[i]);
	    if (i % 4 == 3)
		fputc('\n', stdout);
	}

	if (wlen % 4 != 0)
	    fputc('\n', stdout);

	printf("====================================\n");
    };

    const copper_lite_header & header() const {
	return m_header;
    };

    unsigned int footer() const {
	return m_footer;
    };

    const int len() const {
	return m_len;
    };

    unsigned char * body() const {
	return m_body;
    };
};

class copper_lite {
private:
    addrinfo * m_gai;
    int m_socket;
    FILE * m_fsocket;
    copper_lite_header m_header;
    unsigned int m_footer;
    bool is_opened;
    unsigned char * m_buffer;
    copper_lite_event m_event;

public:

    copper_lite() : m_gai(NULL),
		m_socket(-1),
		m_fsocket(NULL),
		is_opened(false) {
	m_buffer = new unsigned char[EVENT_BUFFER_SIZE];
    };

    copper_lite(const char * host, const char * port) {
	m_buffer = new unsigned char[EVENT_BUFFER_SIZE];
	Open(host, port);
    }

    int Open(const char * host, const char * port) {
	assert(is_opened == false);
	assert(m_gai == NULL);
	assert(m_fsocket == NULL);
	assert(m_socket == -1);

	addrinfo gai_hint;

	memset(&gai_hint, 0, sizeof(gai_hint));

	gai_hint.ai_protocol = PF_INET;
	gai_hint.ai_socktype = SOCK_STREAM;
	gai_hint.ai_protocol = IPPROTO_TCP;

	int gai_error = getaddrinfo(host, port, &gai_hint, &m_gai);

	if (gai_error) {
	    printf("%d %s\n", gai_error, gai_strerror(gai_error));
	    return (1);
	}

	m_socket = socket(m_gai->ai_family, m_gai->ai_socktype, m_gai->ai_protocol);

	int ret = connect(m_socket, m_gai->ai_addr, m_gai->ai_addrlen);

	assert(ret == 0);

	m_fsocket = fdopen(m_socket, "r");

	is_opened = true;

	return 0;
    };

    void Shutdown() {
	shutdown(m_socket, SHUT_RD);
    };

    void Close() {
	if (is_opened) {
	    freeaddrinfo(m_gai);
	    shutdown(m_socket, SHUT_RD);
	    fclose(m_fsocket);
	    close(m_socket);
	    m_socket = -1;
	    m_gai = NULL;
	    m_fsocket = NULL;
	    is_opened = false;
	}
    };

    ~copper_lite() {
	Close();
	delete [] m_buffer;
    };

    void Read() throw(char const *) {
	int ret;

	ret = fread(&m_header, sizeof(m_header), 1, m_fsocket);
	if (ret != 1)
	    throw("reading header");

	if (m_header.magic() != 0x7FFF000A)
	    throw("bad header magic");

	int len = m_header.length();

	if (len % 4 != 0 || len > 1024 * 1024 * 2)
	    throw("bad length in header");

	int wlen = len / 4;

	ret = fread(m_buffer, len, 1, m_fsocket);
	if (ret != 1)
	    throw("reading body");

	ret = fread(&m_footer, sizeof(m_footer), 1, m_fsocket);
	if (ret != 1)
	    throw("reading m_footer");

	if (m_footer & 0x01000000) {
	    printf("****FIFO FULL CONDITION HAPPEN!!! (%08x)****\n", m_footer);
	    // has_error = true;
	}

	m_event.fill(m_header, m_footer, m_buffer, len);
    };

    const copper_lite_header & header() const {
	return m_header;
    };

    const unsigned int footer() const {
	return m_footer;
    };

    const copper_lite_event & Last() {
	return m_event;
    };
};
#endif
