#ifndef	_niki_3n00_h_
#define	_niki_3n00_h_

#include <Rtypes.h>

#include <assert.h>
#include <string.h>
#include <unistd.h>
#include <stdio.h>
#include <errno.h>

#include <netdb.h>
#include <poll.h>

#include <iostream>

extern "C" {

#include "set_vme_module.h"
#include "acs_vme.h"
#include "read_setting.h"
#include "write_setting.h"

int vme_init(char *hostnm,char *portnm);
int quit_vme();
int read_from_sitcp(unsigned char *buff);

};


#define	NIKI_3N00_BUFFER_SIZE (4*4*1024*1024)

class niki_stop_data {
public:
    /* handle big endian data in little endian machine */

    union {
	struct {
	    UChar_t m_module:4;
	    UChar_t m_what:4;
	};
	UChar_t m_stop_id_byte;
    };
    union {
	struct {
	    UChar_t m_channel:5;
	    UChar_t m_last_data:1;
	    UChar_t m_ch_full:1;
	    UChar_t m_reserved:1;
	};
	UChar_t m_info;
    };

    UShort_t m_tdc;
public:

    Int_t GetCh() const {
	return m_channel;
    };

    Int_t GetModule() const {
	return ((m_stop_id_byte >> 4) & 0xF);
	//	return ((m_info >> 4) & 0xF);
    };

    Int_t GetTDC() const {
	return ntohs(m_tdc);
    };

    Bool_t IsChLast() const {
	return (m_info & 0x20) ? 1 : 0;
    };

    Bool_t IsOverflowed() const {
	return (m_info & 0x40) ? 1 : 0;
    };

    UInt_t GetVal() const {
	return *(UInt_t *)(this);
    };

    Bool_t IsUp() const {
	return true;
    };

    Bool_t IsDown() const {
	return false;
    };

    Int_t Module() const {
	return m_module;
    };

    Int_t What() const {
	return m_what;
    };
};

class niki_3n00_header {
public:
    UInt_t m_copper_lite_magic; /* 0x7FFF000A */
    union {
	UInt_t m_module_id;
	struct {
	    UInt_t m_keyword:24;
	    UChar_t m_reserve_1;
	};
	struct {
	    UChar_t m_niki_gain_mode;
	    UChar_t m_niki_mcsr;
	    UChar_t m_niki_module;
	    UChar_t m_reserve_2;
	};
    };

    UInt_t m_length; /* in byte */
    UInt_t m_event_id;
    UInt_t m_finesse_magic; /* 0xFFAA0000 */
    UInt_t m_finesse_event_id; /* lower 8bits are always zero */

public:
    UInt_t copper_lite_magic() const {
	return ntohl(m_copper_lite_magic);
    };

    UInt_t finesse_magic() const {
	return ntohl(m_finesse_magic);
    };

    UInt_t keyword() const {
	return ntohl(m_keyword);
    };

    int Module() const {
	switch (m_niki_module) {
	case 0x01: return 0xA3100;
	case 0x02: return 0xA3200;
	case 0x03: return 0xA3300;
	case 0x04: return 0xA3400;
	default: return -1;
	}
    };

    UInt_t MCSR() const {
	return ntohl(m_niki_mcsr);
    };

    UInt_t length() const {
	return ntohl(m_length);
    };

    UInt_t event_length() const {
	return length() + 4 * sizeof(UInt_t);
    };

    UInt_t event_id() const {
	return ntohl(m_event_id);
    };

    void show(FILE * f = stdout) const {
	fprintf(f, "copper_lite_magic = 0x%08x\n", copper_lite_magic());
	fprintf(f, "module_id         = 0x%08x\n", Module());
	fprintf(f, "length            = 0x%08x\n", length());
	fprintf(f, "event_id          = 0x%08x\n", event_id());
	fprintf(f, "finesse_magic     = 0x%08x\n", finesse_magic());
    };

    int nhit() const {
	return (length() - 20) / 16;
    };

    UInt_t resolution() const {
	return MCSR() & 0xF;
    };
};

class niki_3n00_footer {
public:
    UInt_t m_start;
    UInt_t m_finesse_footer;
    UInt_t m_copper_lite_footer;

public:

    UShort_t start() const {
	return (ntohl(m_start) & 0xFFFF);
    };

    UInt_t finesse_footer() const {
	return ntohl(m_finesse_footer);
    };

    UInt_t copper_lite_footer() const {
	return ntohl(m_copper_lite_footer);
    };

    Bool_t txBufFull() const {
	return (ntohl(m_copper_lite_footer) & 0x01000000) ? 1 : 0;
    };

    void show(FILE * f = stdout) const {
	fprintf(f, "start              = 0x%08x\n", start());
	fprintf(f, "finesse_footer     = 0x%08x\n", finesse_footer());
	fprintf(f, "copper_lite_footer = 0x%08x\n", copper_lite_footer());
    };
};

class niki_3n00_event {
private:
    niki_3n00_header m_header;
    UChar_t m_body[NIKI_3N00_BUFFER_SIZE];

public:
    const niki_3n00_header & header() const {
	return m_header;
    };

    const int event_length() const {
	return header().event_length();
    };

    const uint32_t event_id() const {
	return header().event_id();
    };

    const int nbyte() const {
	return event_length();
    };

    const int nword() const {
	return nbyte() / sizeof(uint32_t);
    };

    const niki_3n00_footer & footer() const {
	char * p = (char *)&m_header;
	return *(niki_3n00_footer *)(p + nbyte() - sizeof(niki_3n00_footer));
    };

    niki_3n00_event(const niki_3n00_event & src) {
	assert(src.nbyte() < sizeof(niki_3n00_event));
	m_header = src.m_header;
	memcpy(m_body, src.m_body, nbyte() - sizeof(m_header));
    };

    void show(FILE * f = stdout) const {

	fprintf(f, "=============niki_3n00_event.show()=======================\n");

	header().show(f);
	fprintf(f, "footer = %08x %08x\n", footer().m_finesse_footer, footer().m_copper_lite_footer);

	fprintf(f, "len = %d (bytes)\n", nbyte());

	assert(nbyte() % 4 == 0);

	int wlen = nword();

	UInt_t * p = (UInt_t *)m_body;

	for (int i=0; i<wlen; i++) {
	    if (i % 4 == 0)
		fprintf(f, "%04x", i*4);
	    fprintf(f, " %08x", ntohl(p[i]));
	    if (i % 4 == 3)
		fputc('\n', f);
	}

	if (wlen % 4 != 0)
	    fputc('\n', f);

	fputs("====================================\n", f);
    };

    const UShort_t start() const {
	return footer().start();
    };

    const int nhit() const {
	return header().nhit();
    };

    const niki_stop_data * const hits() const {
	return (niki_stop_data *)m_body;
    };

#if 0
    UChar_t * body() const {
	return m_body;
    };

    void fill(const niki_3n00_header & header, const niki_3n00_footer & footer, UChar_t * p, int len) {
	assert(len < NIKI_3N00_BUFFER_SIZE);
	m_header = header;
	m_footer = footer;
	m_len = len;
	memcpy(m_body, p, len);
    };
#endif
};

class niki_module_reader {
  int m_module_id;

  char * m_buffer_base;
  char * m_buffer_rp;
  char * m_buffer_wp;

public:

  int bufsize() const {
    return (4 * NIKI_3N00_BUFFER_SIZE);
  };

  niki_module_reader() {
      m_buffer_base = new char[bufsize()];
      m_buffer_rp = m_buffer_base;
      m_buffer_wp = m_buffer_base;
  };

  ~niki_module_reader() {
      delete [] m_buffer_base;
  };

  const niki_3n00_header & header() const {
      return *(niki_3n00_header *)m_buffer_rp;
  };

  bool has_event() const {
      return ((m_buffer_wp - m_buffer_rp) && (header().event_id() != 0xFFFFFFFF));
  };

  const int event_length() const {
      return header().event_length();
  };

  const niki_3n00_footer & footer() const {
    return *(niki_3n00_footer *)(m_buffer_rp + event_length() - sizeof(niki_3n00_footer));
  };

  const niki_3n00_event & event() const {
      return *(niki_3n00_event *)m_buffer_rp;
  };

  int module_id() const {
    return m_module_id;
  };

  int module_id(int i) {
    m_module_id = i;
    return i;
  };

  uint32_t event_number() const {
    return header().event_id();
  };

  void write_event_data(FILE * fp = stdout) const {
    if (has_event()) {
      int ret = fwrite(m_buffer_rp, event_length(), 1, fp);
      assert(ret == 1);
    }
  };

  bool pop() {
    if (has_event()) {
      m_buffer_rp += event().nbyte();
      if (m_buffer_rp == m_buffer_wp) {
	m_buffer_rp = m_buffer_base;
	m_buffer_wp = m_buffer_base;
	return 0;
      } else {
	return 1;
      }
    } else {
      return 0;
    }
  };

  uint32_t fetch_event() {
    if (pop()) {
      return event().event_id();
    }

    read_sitcp(m_module_id);
    int num = read_from_sitcp((unsigned char *)m_buffer_base);
    assert(num >= 36);
    niki_3n00_header * p = (niki_3n00_header *)m_buffer_base;
    assert(p->copper_lite_magic() == 0x7FFF000A);
    assert(p->finesse_magic() == 0xFFAA0000);
    m_buffer_wp = m_buffer_base + num;
    return p->event_id();
  };

  void show_event(FILE * fp = stderr) const {
    if (!has_event()) {
      return;
    }
    uint32_t * p = (uint32_t *)m_buffer_rp;
    const int nword = event().nword();
    fprintf(fp, "nword = %d\n", nword);
    for (int i=0; i<nword; i++) {
      fprintf(fp, "%2d %08x\n", i, ntohl(p[i]));
    }
  };

};

#endif
